/*************************************************** 
It is a classic application that using a Oled Screen to work as a custom cool display watch!

Editor: Lauren
E-Mail: Lauren.pan@dfrobot.com
Date:   2012.11.21
Version:0.1
Description
  Improve the library to work with Oled 9664 from DFRobot and Arduino Uno
Product Link:
  http://www.dfrobot.com/index.php?route=product/product&path=48&product_id=817#.UL7MH009ha0
 ****************************************************/

unsigned char dfrobotIcon[] = {
  0xC0, 0xF0, 0xFC, 0xFC, 0xFE, 0xFE, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x7F, 0x1F,
  0x0F, 0x07, 0x07, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x07, 0x86, 0xCE, 0xFC, 0xFC, 0xF8, 0xE0,
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x07, 0x06, 0x06, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFF, 0xFF, 0x3F, 0x0F, 0x03, 0x03, 0x01, 0x00, 0x00, 0x80, 0xC0, 0xC0, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFE, 0xFE, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0x07, 0x1F, 0x3F, 0x7F, 0x7E, 0x78, 0xF0, 0xE0, 0xC0, 0xC0, 0x80, 0x80, 0x81, 0x80, 0x80, 0xC0,
  0xC0, 0xC0, 0xE0, 0xF0, 0xFC, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x7F, 0x7F, 0x7F, 0x3F, 0x0F
};


#include <Adafruit_GFX.h>
#include <Adafruit_SSD1331.h>
#include <SPI.h>
#include <Metro.h>

Metro flashManager = Metro(500,true);
Metro printManager = Metro(100,true);
Metro lineManager = Metro(30,true);

#define sclk 13
#define mosi 11
#define cs   10
#define rst  9
#define dc   8

// Color definitions
#define	BLACK           0x0000
#define	BLUE            0x001F
#define	RED             0xF800
#define	GREEN           0x07E0
#define CYAN            0x07FF
#define MAGENTA         0xF81F
#define YELLOW          0xFFE0  
#define WHITE           0xFFFF

Adafruit_SSD1331 oled = Adafruit_SSD1331(cs, dc, mosi, sclk, rst);  

int AdjustVal = 0;
int fontWidth = 10;

byte Hour = 8;
byte Minute = 55;
byte Second = 00;
byte lastHour = Hour;
byte lastMinute = Minute;
byte lastSecond = Second;

void setup(void) {
  Serial.begin(9600);
  oled.begin();
  FastClear(0,0,oled.TFTWIDTH-1,oled.TFTHEIGHT-1);
  initAnalog();
  delay(100);

  drawbmpPixel(32,16,dfrobotIcon,32,32);
  
  for(int i = 0; i < 120; i++){
    printStr(30,55,"Loading...",1,HSVtoRGB(i*3,255,255));
  }
  printStr(30,55,"Loading...",1,WHITE);
  delay(1000);
  FastClear(0,0,oled.TFTWIDTH-1,oled.TFTHEIGHT-1);

  drawBorder();
  initClock();
}

uint16_t HSVtoRGB(int hue,int val,int sat){	
  int __max_brightness = 255;
  uint8_t sector = hue / 60;
  uint8_t rel_pos = hue - (sector * 60);
  uint16_t const mmd = 255 * 255; 	/* maximum modulation depth */
  uint16_t top;
  top = val * 255;
  uint16_t bottom = val * (255 - sat); 	/* (val*255) - (val*255)*(sat/255) */
  uint16_t slope = (uint16_t)(val) * (uint16_t)(sat) / 120; 	/* dy/dx = (top-bottom)/(2*60) -- val*sat: modulation_depth dy */
  uint16_t a = bottom + slope * rel_pos;
  uint16_t b = bottom + (uint16_t)(val) * (uint16_t)(sat) / 2 + slope * rel_pos;
  uint16_t c = top - slope * rel_pos;
  uint16_t d = top - (uint16_t)(val) * (uint16_t)(sat) / 2 - slope * rel_pos;

  uint16_t R, G, B;

  if (sector == 0)    {
    R = c;
    G = a;
    B = bottom;
  }
  else if (sector == 1)	{
    R = d;
    G = b;
    B = bottom;
  }
  else if (sector == 2)	{
    R = bottom;
    G = c;
    B = a;
  }
  else if (sector == 3)	{
    R = bottom;
    G = d;
    B = b;
  }
  else if (sector == 4)	{
    R = a;
    G = bottom;
    B = c;
  }
  else	{
    R = b;
    G = bottom;
    B = d;
  }

  uint16_t scale_factor = mmd / __max_brightness;

  R = (uint8_t) (R / scale_factor);
  G = (uint8_t) (G / scale_factor);
  B = (uint8_t) (B / scale_factor);
  return oled.Color565(R,G,B);
}

void drawbmpPixel(unsigned char x,unsigned  char y,unsigned char *image,unsigned char imgSizeX,unsigned char imgSizeY){
  unsigned char row;

  FastClear(x,y,x+imgSizeX,y+imgSizeY);
  if (imgSizeY%8==0) row=imgSizeY/8;  
  else row=imgSizeY/8+1;

  for(int j = 0;j < row; j++){
    for(int i=0; i < imgSizeX; i++){
      for(int k = 0; k < 8 ;k ++){
        if(bitRead(image[i+j*imgSizeX],k) != 0)  oled.drawPixel(x+i,y+j*8+k,CYAN);
      }
    }
  }
}

void initAnalog(){
  pinMode(A1,OUTPUT);
  pinMode(A2,OUTPUT);
  digitalWrite(A1,LOW);
  digitalWrite(A2,HIGH);
}

void initClock(){

  String buffer = convStr(lastHour);
  if(lastHour < 10)  buffer = "0"+buffer;
  printStr(3,20,buffer,3,BLUE);

  buffer = convStr(lastMinute);
  if(lastMinute < 10)  buffer = "0"+buffer;
  printStr(54,20,buffer,3,BLUE);

  printStr(38,20,":",3,BLUE);

}


void loop(){

  if(flashManager.check()){

    static boolean colonState = false;
    if(colonState)  colonState = false;
    else colonState = true;

    if(colonState){
      displayColon();
      displayClock();
    }
    else clearColon();
  }

  if(printManager.check()){

    AdjustVal = analogRead(0);
    AdjustVal = map(AdjustVal,0,1024,0,255);
    Serial.println(AdjustVal);
    static int brightness = 0; 
    if(AdjustVal < 20)  DisplayOFF();
    else if(AdjustVal > 240) DisplayON();
    else if(brightness != AdjustVal){
      DimWindow(brightness);
      brightness = AdjustVal;
    }
    //    Serial.print("FreeRam:");
    //    Serial.println(freeRam());
  }
}

void displayColon(){
  printStr(38,20,":",3,BLUE);
}
void clearColon(){
  FastClear(36,20,52,42);
}

void displayClock(){
  if(Second < 59){
    Second++;
  }
  else{
    Second = 0;
    if(Minute < 59)  Minute++;
    else{
      Minute = 0;
      if(Hour < 23)  Hour++;
      else Hour = 0;
    }
  }

  if(lastSecond != Second){
    lastSecond = Second;
    FastClear(83,10,95,20);
    String buffer = convStr(lastSecond);
    if(lastSecond < 10)  buffer = "0"+buffer;
    printStr(83,10,buffer,1,YELLOW);
  }

  if(lastHour != Hour){
    lastHour = Hour;
    FastClear(3,20,36,42);
    String buffer = convStr(lastHour);
    if(lastHour < 10)  buffer = "0"+buffer;
    printStr(3,20,buffer,3,BLUE);
  }

  if(lastMinute != Minute){
    lastMinute = Minute;
    FastClear(54,20,92,42);
    String buffer = convStr(lastMinute);
    if(lastMinute < 10)  buffer = "0"+buffer;
    printStr(54,20,buffer,3,BLUE);
  }
}

String convStr(int num){
  char str[10];
  itoa(num,str,10);
  return str;
}

void DimWindow(uint8_t brightness){
  oled.writeCommand(0xAB);
  oled.writeCommand(0x00);
  oled.writeCommand(brightness);
  oled.writeCommand(brightness);
  oled.writeCommand(brightness);
  oled.writeCommand(0x00);
  delayMicroseconds(50);
  oled.writeCommand(0xAC);
  delayMicroseconds(50);
}

void DisplayON(){
  oled.writeCommand(0xAF);
  delayMicroseconds(50);
}

void DisplayOFF(){
  oled.writeCommand(0xAE);
  delayMicroseconds(50);
}

void FastClear(uint8_t x0,uint8_t y0,uint8_t x1,uint8_t y1){
  oled.writeCommand(0x25);
  oled.writeCommand(x0);
  oled.writeCommand(y0);
  oled.writeCommand(x1);
  oled.writeCommand(y1);
  delay(1);
}

void ClearWindow(){
  FastClear(0,0,95,63);
  delay(1);
}

void drawBorder(){

  oled.drawLine(0,0,5,5,WHITE);
  oled.drawLine(5,5,90,5,WHITE);
  oled.drawLine(90,5,95,0,WHITE);

  oled.drawLine(0,63,5,58,WHITE);
  oled.drawLine(5,58,90,58,WHITE);
  oled.drawLine(90,58,95,63,WHITE);

}

void printFont(uint8_t x,uint8_t y,char *str,uint8_t s,uint16_t c){
  oled.setCursor(x, y);
  oled.setTextSize(s);
  oled.setTextColor(c);
  oled.print(str);
}

void printStr(uint8_t x,uint8_t y,String str,uint8_t s,uint16_t c){
  oled.setCursor(x, y);
  oled.setTextSize(s);
  oled.setTextColor(c);
  oled.print(str);
}

void setScroll(){
  oled.writeCommand(0x27);
  oled.writeCommand(0x00);
  oled.writeCommand(0x10);
  oled.writeCommand(0x10);
  oled.writeCommand(0x01);
  oled.writeCommand(0x00);
  oled.writeCommand(0x2F);
  delayMicroseconds(10);
}
void disableScroll(){
  oled.writeCommand(0x2E);
  delayMicroseconds(10);
}


int freeRam(void)
{
  extern int  __bss_end; 
  extern int  *__brkval; 
  int free_memory; 
  if((int)__brkval == 0) {
    free_memory = ((int)&free_memory) - ((int)&__bss_end); 
  }
  else {
    free_memory = ((int)&free_memory) - ((int)__brkval); 
  }
  return free_memory; 
} 




